@@ -1,9 +1,10 @@
 # et:ts=4
 # portdpkg.tcl
-# $Id: portdpkg.tcl 26177 2007-06-15 10:11:22Z jmpp@macports.org $
+# $Id: portdpkg.tcl 82192 2011-08-10 08:41:30Z afb@macports.org $
 #
+# Copyright (c) 2005, 2007, 2009, 2011 The MacPorts Project
 # Copyright (c) 2004 Landon Fuller <landonf@macports.org>
-# Copyright (c) 2002 - 2003 Apple Computer, Inc.
+# Copyright (c) 2002 - 2003 Apple Inc.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -14,7 +15,7 @@
 # 2. Redistributions in binary form must reproduce the above copyright
 #    notice, this list of conditions and the following disclaimer in the
 #    documentation and/or other materials provided with the distribution.
-# 3. Neither the name of Apple Computer, Inc. nor the names of its contributors
+# 3. Neither the name of Apple Inc. nor the names of its contributors
 #    may be used to endorse or promote products derived from this software
 #    without specific prior written permission.
 # 
@@ -34,33 +35,36 @@
 package provide portdpkg 1.0
 package require portutil 1.0
 
-set org.macports.dpkg [target_new org.macports.dpkg org.macports.dpkg::main]
+set org.macports.dpkg [target_new org.macports.dpkg portdpkg::main]
 target_runtype ${org.macports.dpkg} always
 target_provides ${org.macports.dpkg} dpkg
-target_requires ${org.macports.dpkg} destroot
+target_requires ${org.macports.dpkg} archivefetch unarchive destroot
 
-# Target Namespace
-namespace eval org.macports.dpkg {
+namespace eval portdpkg {
 }
 
 # Options
+options dpkg.asroot
 options package.destpath
 
+# Set up defaults
+default dpkg.asroot yes
+
 set_ui_prefix
 
-proc org.macports.dpkg::main {args} {
-	global UI_PREFIX destpath os.arch os.platform
+proc portdpkg::main {args} {
+	global UI_PREFIX destpath os.arch os.platform supported_archs configure.build_arch
     
-	ui_msg "$UI_PREFIX [format [msgcat::mc "Creating dpkg for %s-%s"] [option portname] [option portversion]]"
+	ui_msg "$UI_PREFIX [format [msgcat::mc "Creating dpkg for %s-%s"] [option subport] [option version]]"
 
 	# get deplist
-	set deps [make_dependency_list [option portname]]
+	set deps [make_dependency_list [option subport]]
 	set deps [lsort -unique $deps]
 	foreach dep $deps {
 		set name [lindex [split $dep /] 0]
 		set vers [lindex [split $dep /] 1]
 		# don't re-package ourself
-		if {$name != [option portname]} {
+		if {$name != [option subport]} {
 			lappend dependencies "${name} (>= ${vers})"
 		}
 	}
@@ -75,7 +79,7 @@ proc org.macports.dpkg::main {args} {
 			return -code error [format [msgcat::mc "Can not create dpkg control directory. %s not a directory."] ${controlpath}]
 		} else {
 			ui_info [msgcat::mc "Removing stale dpkg control directory."]
-			system "rm -rf \"${controlpath}\""
+			delete "${controlpath}"
 		}
 	}
 	file mkdir ${controlpath}
@@ -94,9 +98,9 @@ proc org.macports.dpkg::main {args} {
 	
 	# Create dpkg version number
 	if {[expr [option epoch] != 0]} {
-		set pkg_version "[option epoch]:[option portversion]"
+		set pkg_version "[option epoch]:[option version]"
 	} else {
-		set pkg_version "[option portversion]"
+		set pkg_version "[option version]"
 	}
 	if {[expr [option revision] != 0]} {
 		append pkg_version "-[option revision]"
@@ -111,7 +115,7 @@ proc org.macports.dpkg::main {args} {
 	} elseif {[exists description]} {
 		set pkg_long_description " [option description]\n"
 	} else {
-		set pkg_long_description " [option portname]\n"
+		set pkg_long_description " [option subport]\n"
 	}
 
 	if {[exists homepage]} {
@@ -127,8 +131,9 @@ proc org.macports.dpkg::main {args} {
 	# sparc and sparc64. The operating system, os, is one of: linux, gnu,          
 	# freebsd and openbsd. Use of gnu in this string is reserved for the           
 	# GNU/Hurd operating system.
-	switch -regex ${os.arch} {
+	switch -regex ${configure.build_arch} {
 		i[3-9]86 { set pkg_arch "i386" }
+		x86_64 { set pkg_arch "x86_64" }
 		default { set pkg_arch ${os.arch} }
 	}
 
@@ -136,9 +141,16 @@ proc org.macports.dpkg::main {args} {
 	# the operating system name
 	if {${os.platform} != "linux"} {
 		set pkg_arch "${os.platform}-${pkg_arch}"
+	} elseif {${pkg_arch} == "x86_64"} {
+		set pkg_arch "amd64"
+	}
+	
+	# An architecture-independent package
+	if {$supported_archs == "noarch"} {
+		set pkg_arch "all"
 	}
 
-	puts $controlfd "Package: [option portname]"
+	puts $controlfd "Package: [option subport]"
 	puts $controlfd "Architecture: ${pkg_arch}"
 	puts $controlfd "Version: ${pkg_version}"
 	puts $controlfd "Section: ${pkg_category}"
@@ -157,15 +169,15 @@ proc org.macports.dpkg::main {args} {
 	system "dpkg-deb -b \"${destpath}\" \"[option package.destpath]\""
 
 	ui_info [msgcat::mc "Removing dpkg control directory."]
-	system "rm -rf \"${controlpath}\""
+	delete "${controlpath}"
 }
 
-proc org.macports.dpkg::make_dependency_list {portname} {
+proc portdpkg::make_dependency_list {portname} {
 	set result {}
-	if {[catch {set res [mport_search "^$portname\$"]} error]} {
+	if {[catch {set res [mport_lookup $portname]} error]} {
 		global errorInfo
 		ui_debug "$errorInfo"
-		ui_error "port search failed: $error"
+		ui_error "port lookup failed: $error"
 		return 1
 	}
 	foreach {name array} $res {
